#!/etc/opt/openit/perl

# Adds the following filters to all licpoll2 filter files (inserted after the License2Action filter):
# Mapping:RLM-Sharing()
# CollapseEvents:RLM-sharing()
use File::Basename;

my @filters_to_skip = (
	"license-total-usage.conf",
	"license-hostgroups.conf",
	"license-host-usage.conf",
	"license-usergroups.conf",
	"license-individual-usage.conf"
);

my @mapping_def = (
	"\t\tMapping:RLM-Sharing()=\"\"\n",
	"\t\t{\n",
	"\t\t\tinactive(Bool)=\"false\"{}\n",
	"\t\t\tfilter(String)=\"Mapping\"{}\n",
	"\t\t\tlimiting(String)=\"\$data_type == 'RLM'\"{}\n",
	"\t\t\tmap-file(FileName)=\"\${OpeniT.directories.conf}/rlm-shared-licenses.map\"{}\n",
	"\t\t\tset-value(String)=\"#1:var5=\$feature . '\\\\' . \$daemon . '\\\\' . \$product\"{}\n",
	"\t\t\tsource(String)=\"var5\"{}\n",
	"\t\t\ttarget(String)=\"var6\"{}\n",
	"\t\t}\n"
);

my @collapse_def = (
	"\t\tCollapseEvents:RLM-sharing()=\"\"\n",
	"\t\t{\n",
	"\t\t\tinactive(Bool)=\"false\"{}\n",
	"\t\t\tfilter(String)=\"CollapseEvents\"{}\n",
	"\t\t\tsignature(String)=\"product,feature,feature_version,licenses,host,user\"{}\n",
	"\t\t\ttime-window-size(TimeSpan)=\"PT1H\"{}\n",
	"\t\t\tlimiting(String)=\"\$var6 == 'UH'\"{}\n",
	"\t\t\tnew-time(String)=\"first\"{}\n",
	"\t\t}\n"
);

my @filters = get_filters_to_update( "/var/opt/openit/etc/filtrator", "^license-", "flex-log|inventory|-meter-" );
push @filters, get_filters_to_update( "/var/opt/openit/etc/filtrator", "olap-", "winapp|freeze" );

# Traverse each filter file and insert filter process and definition
foreach my $filter ( @filters ) {
	update_filter(  $filter );
}



# Subroutines -----------

sub update_filter {
	my ( $filter ) = @_;
	
	my $filtername = basename($filter);

	if( ! -e $filter ) {
		print "Failed to detect '$filtername'; Skipping to next filter file...\n";
		return;
	}

	my @filter_lines = read_file( $filter );
	
	if ( !grep { /licpoll2/ } @filter_lines ) {
		print " -- '" . $filtername ."' is not licpoll2 skipping...\n";
		return;
	}
	
	if ( grep( /$filtername/, @filters_to_skip ) ) {
		print " -- '" . $filtername ."' is an old version skipping...\n";
		return;
	}
	

	# Check if filter file has already been updated
	if( grep { /Mapping:RLM-Sharing/ } @filter_lines ) {
		print " -- '" . $filtername . "' is already updated. Moving no next filter file...\n";
		return;
	}

	my $index = 0;

	# Traverse to location after License2Action filter process
	$index = locate_line( $index, "License2Action", \@filter_lines );
	$index++;
	
	# Insert new filter processes
	@filter_lines[$index] =~ s/{\n/{\n\t\tMapping:RLM-Sharing()=""\n\t\t{\n\t\tCollapseEvents:RLM-sharing()=""\n\t\t{\n/;

	# Insert closing curly brackets for new filter processes
	$index = locate_line( $index, "^\t}\n", \@filter_lines );
	@filter_lines[$index] =~ s/}\n/}\n\t}\n\t}\n/;

	# Traverse to location after filter-defs line
	$index = locate_line( $index, "filter-defs", \@filter_lines );
	$index += 2;

	# Insert arrays for new filter definitions
	splice @filter_lines, $index, 0, @collapse_def;
	splice @filter_lines, $index, 0, @mapping_def;

	save_file( $filter, 2171, @filter_lines );
	print " --- INFO: Done updating '$filtername'\n";
}

sub get_filters_to_update {
    my ( $filter_dir, $include, $exclude ) = @_;

	opendir( DIR,$filter_dir );
	my @tmpfilters = readdir( DIR );
	closedir( DIR );
	
	@tmpfilters = grep { /\.conf$/ } @tmpfilters;
	@tmpfilters = grep { /$include/ } @tmpfilters;
	@tmpfilters = grep { !/old/ } @tmpfilters;
	
    if( defined( $exclude ) && $exclude ne "" ) {
		@tmpfilters = grep { !/$exclude/ } @tmpfilters;
    }
	
	my @filters;

	foreach my $file ( @tmpfilters ) {
        push @filters, "$filter_dir/$file";
    }
    
	return @filters;
}

sub read_file {
	my ( $cfg ) = @_;
	open( IN, "$cfg" ) || die "ERROR: Unable to read cfg '$cfg'!\n";
	my @lines = <IN>;
	close( IN );
	return @lines;
}

sub locate_line {
	# For carets and colons, use the . character to represent them
	# For the 3rd argument, always add a \ as prefix to the array
	my ( $start, $find, $lines_ref ) = (@_);
	my ( $index ) = grep { @$lines_ref[$_] =~ /$find/ } $start ..@$lines_ref;
	return $index;
}

sub save_file {
	my ( $cfg, $patch, @lines ) = @_;
	my $old = "$cfg.pre_$patch";
	my $new = "$cfg.new";
	
	open( OUT, ">$new" ) || die "ERROR: Can't write new cfg '$new'!\n";
	print OUT @lines;
	close( OUT );
	
	if( ! rename( $cfg, $old ) ) {
		die "ERROR: Unable to rename previous config '$cfg' to '$old'!\n";
	}
	
	if( ! rename( $new, $cfg ) ) {
		die "ERROR: Unable to rename new config to '$cfg'!\n";
	}
}
